/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.sftp.connection;

import java.io.IOException;

import com.ebmwebsourcing.easycommons.lang.StringHelper;
import com.sshtools.j2ssh.SftpClient;
import com.sshtools.j2ssh.SshClient;
import com.sshtools.j2ssh.authentication.AuthenticationProtocolState;
import com.sshtools.j2ssh.authentication.SshAuthenticationClient;
import com.sshtools.j2ssh.transport.IgnoreHostKeyVerification;

/**
 * This class handle a connection to the SFTP server with the specified configuration parameters.
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class BasicSftpClient extends SshClient {

    /**
     * Sftp client instance
     */
    protected SftpClient client;

    /**
     * Information on the SFTP server.
     */
    protected final SFTPConnectionInfo info;

    public BasicSftpClient(final SFTPConnectionInfo connectionConfiguration) {
        super();
        this.info = connectionConfiguration;
    }

    /**
     * Change the path on the remote server.
     * 
     * @throws IOException
     */
    public void configure() throws IOException {
        if (!StringHelper.isNullOrEmpty(info.getDirectory())) {
            this.client.cd(info.getDirectory());
        }
    }

    /**
     * Open a ssh connection on the remote server and open a sftpclient.
     * 
     * @throws SshAuthenticationException
     * @throws IOException
     */
    public void connectAndLog() throws IOException {
        this.setSocketTimeout((int) this.info.getMaxIdleTime());
        try {
            this.connect(info.getServer(), info.getPort(), new IgnoreHostKeyVerification());
        } catch (IOException ex) {
            final StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't open a connection to [");
            errorSB.append(info);
            errorSB.append("]. Cause : " + ex.getMessage());
            throw new IOException(errorSB.toString(), ex);
        }
        // Retrieve the authentication informations
        final SshAuthenticationClient sshAuthenticationClient;
        sshAuthenticationClient = info.getSshAuthentication().getAuthenticationClient();
        // Try the authentication
        int authenticationResult = this.authenticate(sshAuthenticationClient);
        if (authenticationResult != AuthenticationProtocolState.COMPLETE) {
            this.disconnect();
            throw new IOException("Can't open a connection to [" + info
                    + "]. Cause : Authentication failed");
        }
        this.client = this.openSftpClient();
    }

    public SFTPConnectionInfo getInfo() {
        return info;
    }
}
